// Copyright (c) 2010, SoftLayer Technologies, Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

/*
    ProgressHUD
    
    A view that pops up to indicate when the widget is busy.
    Typically this is shown when the widget is requesting
    information from the server.
    
    The two primary methods on this object that are of interest
    are the beginActivityWithContextAndTitle and endActivityWithContext
    methods.
    
    beginActivityWithContextAndTitle - Call this to tell the progress
    indicator that some activity has begun. The "context" is a string you
    will use to identify the activity (e.g. "Downloading_File"). The title
    is the string that is presented to the user while the activity is
    happening.
    
    endActivityWithContext - Call this with the same context string you passed to
    beginActivityWithContextAndTitle once an activity is complete.
    
    The HUD is displayed automatically while activities are in progress.  Correspondingly 
    it is hidden when all activities have completed.
    
*/

ProgressHUD = function ()
{
    this.statusView = document.createElement("div");
    this.statusView.setAttribute("class", "progressHUD");
    this.statusView.setAttribute("id", "progressHUD");
    this.statusView.style.webkitTransition = "opacity 0.2s default";
    
    this.statusTextLabel = document.createElement("span");
    this.statusTextLabel.setAttribute("class", "progressHUDLabel");
    this.statusTextLabel.innerText = "ProgressHUD";
    this.statusView.appendChild(this.statusTextLabel);

    document.body.appendChild(this.statusView);
}

ProgressHUD.prototype.beginActivityWithContextAndTitle = function(activityContext, activityTitle)
{
    if(null == activityContext || null == activityTitle)
    {
        throw TypeError
    }
    
    // Create and observe an activity tracker.  This is a helper object which
    // keeps track of the list of activities that are currently underway.
    if(!this.activityTracker)
    {
        var self = this;
        this.activityTracker = new SLActivityTracker();
        this.activityTracker.addObserver(this, 
            function (activityTracker, activityTrackerConstant, activityContext) 
            { 
                self.activityTrackerNotification(activityTracker, activityTrackerConstant, activityContext) 
            });
    }

    // Tell the activity tracker about the new activity.
    this.activityTracker.beginActivityWithContextAndTitle(activityContext, activityTitle);
    
    // If this is the first activity there will not be a show timer.  The timer waits
    // for a half a second before actually putting the HUD up on the screen. (activities
    // that begin and end within a half second won't actually display the HUD).
    if(!this.showTimer)
    {
        var self = this;
        this.showTimer = setTimeout(function () { self.showTimerFired() }, 500);
    }
}

ProgressHUD.prototype.endActivityWithContext = function(activityContext)
{
    if(!activityContext)
    {
        throw TypeError
    }

    // Tell the activity tracker that we're done with the task.
    this.activityTracker.endActivityWithContext(activityContext);
    
    // If the activity tracker knows of another activity that's
    // going on, change the status text to that activity.
    //
    // If there are no other activities, then hide the HUD.
    if(null == this.activityTracker.currentActivityTitle())
    {
        // No more activites.  Clear the showTimer if one is still pending
        // and hide the HUD.
        if(this.showTimer)
        {
            clearTimeout(this.showTimer);
            this.showTimer = null;
        }
        
        this.hideStatusView()
    } else {
        this.updateStatusText();
    }
}

ProgressHUD.prototype.showStatusView = function()
{
    // Make sure the opacity is 0 (setting the transition time to 0 so it happens immediately)
    this.statusView.style.webkitTransitionDuration = "0s";
    this.statusView.style.opacity = 0;
    
    // Show the status view (although it will still be completely transparent)
    this.statusView.style.visibility = "visible";

    // Center the view in the window
    this.statusView.style.left = "" + (window.innerWidth - this.statusView.clientWidth) / 2.0 + "px";

    // Change the transition duration back to 0.2s and make the view visible.
    this.statusView.style.webkitTransitionDuration = "0.2s";
    this.statusView.style.opacity = 1;
}

ProgressHUD.prototype.hideStatusView = function ()
{
    var self = this;
    this.statusView.style.webkitTransitionDuration = "0.2s";
    this.statusView.style.opacity = 0;

    setTimeout(function () { self.statusView.style.visibility = "hidden"; }, 200);
}

ProgressHUD.prototype.updateStatusText = function ()
{
    var newStatusText = null;
    
    if(this.activityTracker)
    {
        newStatusText = this.activityTracker.currentActivityTitle();
    }
    
    if(newStatusText && newStatusText.length > 0)
    {
        this.setStatusText(newStatusText);
    }
}

ProgressHUD.prototype.activityTrackerNotification = function(activityTracker, activityTrackerConstant, activityContext)
{
    this.updateStatusText();
}

ProgressHUD.prototype.showTimerFired = function()
{
    this.showStatusView();
    
    clearTimeout(this.showTimer);
    this.showTimer = null;
}

ProgressHUD.prototype.setStatusText = function (newText)
{
    // Change the text and re-center the view
    this.statusTextLabel.innerText = newText;
    this.statusView.style.left = "" + (window.innerWidth - this.statusView.clientWidth) / 2.0 + "px";
}