// Copyright (c) 2010, SoftLayer Technologies, Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

/*
    SLAPIRequest
    
    A class that manages the submission of a request to the SoftLayer portal API.
    This particular class uses the REST interface.
    
    Requests are handled asynchronously and invoke callbacks that you supply when they complete.
    
    The SLAPIRequest object has special behavior for handling JSON responses from the server. If the 
    request succeeds and the server identifies the returned data as JSON, the request object will try 
    to parse the JSON. If that parsing fails, OR if the returned JSON object represents a server 
    error, the failure callback will be invoked.  If parsing succeeds, the objects parsed from the JSON 
    are stored in the request's "responseObject" property (where they can be retrieved by code in the
    success callback).


    Initialization parameters include:
    
    accountUser - an instance of SLAccountUser giving the username and API key for the request.
    serviceName - The name of the service to invoke. (e.g. "SoftLayer_Account", see http://sldn.softlayer.com/wiki/index.php/Category:API_Services)
    serviceMethod - The method of the service you would like to invoke.  e.g. "getOpenTickets"
    objectID - A particular object ID passed to the request.

	After you've constructed one you should set the following properties:
    
	successCallback - a function with the signature function(request) which will be invoked when the 
                        request succeeds.                          

	failureCallback - a function with the signature function(request, exception) which will be invoked 
                        if the request fails. The exception parameter indicates why
                        the request failed. 
                        
                        If the server reports an API error as a JSON object, the
                        exception will be SLAPIRequest.apiCallResponseError and the "responseObject"
                        property of the request will be the JSON "error" object returned.

	You can optionally add an object mask by setting:
    objectMask - An array of strings to be sent as the objectMask for the request.    
*/
	
function SLAPIRequest(accountUser, serviceName, serviceMethod, objectID)
{
	if(null == accountUser)
		throw new Error("APIRequests require an account user")
		
	if(null == serviceName || '' == serviceName )
		throw new Error("API requests require a service name")

	if(null == serviceMethod)
		throw new Error("API Request require a service method")

    this.serviceName = escape(serviceName.trim());
    this.serviceMethod = escape(serviceMethod.trim());
    this.objectID = objectID;

	this.accountUser = accountUser

	this.httpMethod = "GET"
	this.httpBody = null
	this.successCallback = function(request) {}
	this.failureCallback = function(request, exception) {}

    return this;
}

SLAPIRequest.prototype.execute = function ()
{
	var httpRequest = new XMLHttpRequest()
	try {
		var self = this;

        // Uncomment to log requests and their URLS as they are sent.
        // console.log("Issuing request for " + this.apiCallURL())

		httpRequest.open(this.httpMethod,
							this.apiCallURL(),
							true,
							this.accountUser.username,
							this.accountUser.apiKey)
		httpRequest.onreadystatechange = function() { self.readyStateChanged(httpRequest) }
        httpRequest.setRequestHeader("Cache-Control", "no-cache");
		httpRequest.send(this.httpBody)
	}

	catch(exception)
	{    
        if(this.failureCallback instanceof Function)
        {
            this.failureCallback(httpRequest, exception)
        }
	}
}

// Piece together a URL that represents a call to the REST API using the fields in
// the request object.
SLAPIRequest.prototype.apiCallURL = function ()
{	
	var apiPath = this.serviceName.stringByAppendingPathComponent(this.serviceMethod)
	if(null != this.objectID)
	{
		apiPath = apiPath.stringByAppendingPathComponent(objectID.toString())
	}

    if(this.objectMask)
    {
        apiPath = apiPath + "?objectMask=" + this.objectMask.join(";")
    }

	var apiCallURL = softlayer.API_BASE_URL.stringByAppendingPathComponent(apiPath)

	return apiCallURL;
}

// If the response from the server is a JSON object that is successfully parsed
// Check to see if it's a server-side application error and call the appropriate
// callback (success or failure) depending on that check.
SLAPIRequest.prototype.handleJSONResponse = function (httpRequest, jsonObject)
{
    // Store the JSON object in the request where the callbacks can find it.
    httpRequest.responseObject = jsonObject;

    // If it's an error request, call the failure callback,
    // otherwise call the success callback.
    if(jsonObject["error"] === undefined)
    {
        // A json object was successfully retrieved!
        if(this.successCallback instanceof Function)
        {
            this.successCallback(httpRequest)
        }
    } else {
        // Looks like the API reported a failure.
        if(this.failureCallback instanceof Function)
        {
            this.failureCallback(httpRequest, SLAPIRequest.apiCallResponseError)
        }
    }
}

// Called when a request completes without any kind of network errors.
SLAPIRequest.prototype.handleSuccessfulRequest = function (httpRequest) 
{
    // Determine if there is a JSON response... if so, try to parse
    // the JSON.  Otherwise just call the success callback.
    var responseType = httpRequest.getResponseHeader("Content-Type");
    if("application/json" == responseType)
    {
        try
        {
            var responseObject = JSON.parse(httpRequest.responseText);
            if(responseObject)
            {
                this.handleJSONResponse(httpRequest, responseObject)
            }
        }

        catch(e)
        {
            if(this.failureCallback instanceof Function)
            {
                this.failureCallback(httpRequest, e)
            }
        }
    } else {
        // Response didn't report itself as JSON so just try to call
        // the generic success callback.
        if(this.successCallback instanceof Function)
        {
            this.successCallback(httpRequest)
        }
    }            
}

SLAPIRequest.prototype.readyStateChanged = function (httpRequest) 
{
    if(httpRequest.DONE == httpRequest.readyState)
    {
        if(httpRequest.status >= 200 && httpRequest.status <= 299)
        {
            this.handleSuccessfulRequest(httpRequest);
        } else {
            // Response wasn't in the 200's so this must be come kind of networking
            // or server error.
            if(this.failureCallback instanceof Function)
            {
                this.failureCallback(httpRequest, SLAPIRequest.httpResponseError)
            }
        }
    }
}

SLAPIRequest.httpResponseError = new Error("The server request did not complete because of an error.")
SLAPIRequest.apiCallResponseError = new Error("The server-side application reported an error.")
