// Copyright (c) 2010, SoftLayer Technologies, Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

/*
    SLAccountUser
    
    An object class that represents the credentials settings
    for an Account user in the softlayer system.
    
    The primary properties of an instance of this class are the
    username and API key of the account.
    
    The object extends the KVO class so that it can participate
    in bindings.
    
    This class has a method, "validate" which tries to make sure
    that the username and password of the object are recognized
    by the SoftLayer API.
    
    The class also makes provisions to save and restore a single
    instace of the class from the widget's preferences.
*/

SLAccountUser = Class.create(DC.KVO, {
    constructor : function(username, apiKey)
    {
        this.username = username;
        this.apiKey = apiKey
        this.hasBeenValidated = false;
    },

    saveInPreferences : function ()
    {
        widget.setPreferenceForKey(this.username, createInstancePreferenceKey("softlayer_username"));
        widget.setPreferenceForKey(this.apiKey, createInstancePreferenceKey("softlayer_apiKey"));
    },

    removeFromPreferences : function ()
    {
        widget.setPreferenceForKey(null, createInstancePreferenceKey("softlayer_username"));
        widget.setPreferenceForKey(null, createInstancePreferenceKey("softlayer_apiKey"));    
    },

    // If either the username or apiKey changes... be sure to note that
    // the validation of the user is no longer certain.
    didChangeValueForKey : function (key)
    {
        if(key == "username" || key == "apiKey")
        {
            this.hasBeenValidated = false;
        }

        this.base(key);
    },
    
    // Validate the user credentials.  Once validated, call the callback with true
    // if the user is valid, and false otherwise. The callback should have the signature:
    //
    // function (userIsValidBoolean)
    //  
    validate : function (userIsValidCallback)
    {
        if(this.username && this.apiKey && 0 != this.username.length && 0 != this.apiKey.length)
        {
            var self = this;
            var apiRequest = new SLAPIRequest(this, "SoftLayer_Account", "getObject.json", null)

            apiRequest.successCallback = function (httpRequest) {
                self.hasBeenValidated = true
                if(userIsValidCallback instanceof Function)
                {
                    userIsValidCallback(true)
                }
            }

            apiRequest.failureCallback = function (httpRequest, exception) {
                self.hasBeenValidated = false;
                if(userIsValidCallback instanceof Function)
                {
                    userIsValidCallback(false)
                }
            }

            apiRequest.execute()
        } else {
            this.hasBeenValidated = false;

            // the user is invalid
            if(userIsValidCallback instanceof Function)
            {
                userIsValidCallback(false)
            }
        }
    }
});

// This is a "class method" it tries to create an instance of 
// SLAccountUser from the widget's preferences.
SLAccountUser.userFromPreferences = function ()
{
    var user_from_prefs = null;
    var username = widget.preferenceForKey(createInstancePreferenceKey("softlayer_username"));
    var api_key = widget.preferenceForKey(createInstancePreferenceKey("softlayer_apiKey"));

    if(username && api_key)
    {
        user_from_prefs = new SLAccountUser(username, api_key);
    }

    return user_from_prefs;
}
