// Copyright (c) 2010, SoftLayer Technologies, Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

/*
    SLActivityTracker
    
    A simple class that keeps up with a list of "activities".  Each activity
    is identified by its "context", a string that should be a unique in the context
    of the activity tracker.

    Along with each activity there is a "title".  The activity trackier keeps
    track of the list of activities, and their titles.  It can return the title of
    one activity, the one it considers to be the "current" activity.
    
    The activity tracker also includes a Observer mechanism so others
    are notified when activities are started, or complpeted.
*/

SLActivityTracker = function ()
{
    this.activities = new Hash();
    this.observers = new Array();
}

SLActivityTracker.ActivityWillBegin = "ActivityWillBegin"
SLActivityTracker.ActivityEnded = "ActivityEnded"

SLActivityTracker.prototype.currentActivityTitle = function ()
{
    var currentTitle = null;

    if(this.activities && this.activities.length() > 0)
    {
        var firstKey = this.activities.keys()[0];
        currentTitle = this.activities.objectForKey(firstKey);
    }
    
    return currentTitle;
}

SLActivityTracker.prototype.beginActivityWithContextAndTitle = function (activityContext, activityTitle)
{
    if(null == activityContext || null == activityTitle)
    {
        throw TypeError
    }

    this.activities.setObjectForKey(activityTitle, activityContext);
    this._NotifyObservers(SLActivityTracker.ActivityWillBegin, activityContext);
}

SLActivityTracker.prototype.endActivityWithContext = function (activityContext)
{
    if(null == activityContext)
    {
        throw TypeError
    }

    this.activities.setObjectForKey(null, activityContext);
    this._NotifyObservers(SLActivityTracker.ActivityEnded, activityContext);
}

SLActivityTracker.prototype._NotifyObservers = function(notification, context)
{
    for(index = 0; index < this.observers.length; index++)
    {
        var observation = this.observers[index];
        observation.callback(this, notification, context);
    }   
}

// Callback should be of the form:
//
// function(activityTracker, ActivityTrackerNotificationConstant, activityTrackerContext)
//
// The first argument to the callback will be the activity tracker that is sending the notification.
// The second paramter will be one of:
//
// SLActivityTracker.ActivityWillBegin
// SLActivityTracker.ActivityEnded
// 
// The third paramter will be the context of the activity that has begun or ended.
SLActivityTracker.prototype.addObserver = function (newObserver, callback)
{
    if(null == newObserver || null == callback || !(callback instanceof Function))
    {
        throw TypeError;
    }

    var observation = { observer: newObserver, callback: callback };
    this.observers.push(observation);
}

SLActivityTracker.prototype.removeObsever = function (observerToRemove)
{
    var indexesToRemove = [];
    for(index in this.observers)
    {
        var observation = this.observers[index];
        if(observation.observer == observerToRemove)
        {
            indexesToRemove.push(index);
        }
    }
    
    while(indexesToRemove.length > 0)
    {
        this.observers.splice(indexesToRemove.pop(), 1);
    }
}