// Copyright (c) 2010, SoftLayer Technologies, Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

/*
    TicketsWidget

    A controller that represents the entire widget as a whole.  It coordinates
    the widget life cycle, handles the "flippedness" of the widget and other
    mundane operational concerns. It does this with the help of two other 
    controllers (one for the front, and one for the back)
    
    This object is also responsible for retrieving and keeping track of
    the ticket's list as returned from the server.  It exposes the ticket
    list using KVO.
*/
    
TicketsWidget = Class.create(DC.Controller, 
{
    exposedBindings: ["openTickets"],

    constructor : function ()
    {
        this.flippedToFront = true;    // we assume we start with the front showing
    },

    refreshTicketList : function ()
    {
        var self = this;

        // If the current user has not been validated then try to validate the
        // user before fetching the tickets
        if(!this.currentUser.hasBeenValidated)
        {
            this.currentUser.validate(
                function (userIsValid) {
                    if(userIsValid)
                    {
                        self.currentUser.saveInPreferences()

                        // ask for a ticket list refresh again at the earliest opportunity
                        setTimeout(function () { self.refreshTicketList(); }, 0)
                    } else {
                        setTimeout(function () { self.flipToBack(); }, 0)
                    }
                });
        } else {
            // Create a new request asking for the current account's open tickets.
            var apiRequest = new SLAPIRequest(this.currentUser, 
                                                "SoftLayer_Account", 
                                                "OpenTickets.json", null)

            // Identify what properties of the tickets we want to know about.
            apiRequest.objectMask = ["id", "title", "modifyDate", "awaitingUserResponseFlag", "serviceProvider"]

            // set self for the benefit of the callbacks
            var self = this

            apiRequest.successCallback = function (httpRequest) {
                self.progressHUD.endActivityWithContext("fetchingOpenTickets");

                // The response should be the list of open tickets
                self.setValueForKey(httpRequest.responseObject, "openTickets");
            }

            apiRequest.failureCallback = function (httpRequest) {
                self.progressHUD.endActivityWithContext("fetchingOpenTickets");
                self.widgetFrontController.setTicketListStatus("A communication error kept the list of tickets from being retrieved.");
            }

            self.progressHUD.beginActivityWithContextAndTitle("fetchingOpenTickets", "Fetching Open Tickets…")

            apiRequest.execute()
        }
    },

    load : function () 
    {
        /* Create the progress popup window */
        this.progressHUD = new ProgressHUD()

        /* Try to restore the current user from the preferences */
        var user_from_prefs = SLAccountUser.userFromPreferences()
        if(user_from_prefs)
        {
            this.currentUser = user_from_prefs
        } else {
            // set the current user to an invalid user
            this.currentUser = new SLAccountUser("", "")
        }

        // Create two sub-controllers (one for the controls on the front side of the
        // widget, and a similar one for the back side).  Notify them both of the
        // fact that the widget was loaded.
        this.widgetFrontController = new TicketsWidgetFrontController()
        this.widgetFrontController.load()

        this.widgetBackController = new TicketsWidgetBackController()
        this.widgetBackController.load()
    },

    remove : function () 
    {
        if(this.currentUser)
        {
            this.currentUser.removeFromPreferences();
        }
    },

    show : function () 
    {
        if(this.flippedToFront)
        {
            this.refreshTicketList();
            this.widgetFrontController.show();
        } else {
            this.widgetBackController.show();
        }
    },

    hide : function () 
    {
        if(this.flippedToFront)
        {
            this.widgetFrontController.hide();
        } else {
            this.widgetBackController.hide();
        }
    },

    // Flip the widget over to its back side, notifying the 
    // sub-controllers of the hidings and showings
    flipToBack : function ()
    {
        this.widgetFrontController.hide();
        
        var front = document.getElementById("front")
        var back = document.getElementById("back")

        if (window.widget) 
        {
            widget.prepareForTransition("ToBack")
        }

        front.style.display = "none"
        back.style.display = "block"

        if (window.widget) 
        {
            setTimeout(function () { widget.performTransition() }, 0)
        }

        // Be sure the back controller knows he's showing
        var self = this;
        setTimeout(function () { self.flippedToFront = false; self.widgetBackController.show() }, 1)
    },

    // Flip the widget over to its front side, notifying the 
    // sub-controllers of the hidings and showings    
    flipToFront : function () 
    {
        this.widgetBackController.hide()

        var front = document.getElementById("front")
        var back = document.getElementById("back")

        if (window.widget) 
        {
            widget.prepareForTransition("ToFront")
        }

        front.style.display = "block"
        back.style.display = "none"

        if (window.widget) 
        {
            setTimeout( function () { widget.performTransition() }, 0)
        }

        // Be sure the front controller knows he's showing
        var self = this;
        setTimeout(function () { self.flippedToFront = true; self.widgetFrontController.show(); self.didFlipToFront() }, 1)
    },
    
    // Called once the frontward flip is complete.
    didFlipToFront : function ()
    { 
        this.refreshTicketList();
    }
});