// Copyright (c) 2010, SoftLayer Technologies, Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//  * Neither SoftLayer Technologies, Inc. nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

/*
    TicketsWidgetFrontController
    
    A controller managing the controls on the front side of the widget.
*/
TicketsWidgetFrontController = Class.create(DC.Controller, 
{
    load : function()
    {
        /* Turn the refresh button div into a refresh button view */
        var refreshButtonDiv = document.getElementById("refresh_button");
        if(refreshButtonDiv)
        {
            this.refreshButton = new RefreshButton(refreshButtonDiv);
            refreshButtonDiv.onclick = function () { softlayer.ticketsWidget.refreshTicketList() }
        }

        this.setLastUpdateStatus("");
        if(softlayer.ticketsWidget)
        {
            // observe when the widgets gets a new ticket list
            softlayer.ticketsWidget.addObserverForKeyPath(this, this.openTicketsChanged, "openTickets", null)
        }
    },
    
    show : function ()
    {
    },

    hide : function ()
    {
    },
    
    openTicketsChanged : function (changeNotification, keyPath, context)
    {
        // The list of open tickets known to the widget has changed.
        // Remove the current tickets from the list and add new
        // rows for the current batch of tickets
        if(softlayer.ticketsWidget)
        {
            var openTickets = softlayer.ticketsWidget.valueForKey("openTickets");
            if(openTickets)
            {
                if(openTickets.length != 0)
                {
                    this.clearTicketList()
                    this.addTicketsToList(openTickets)
                } else {
                    this.setTicketListStatus("No open tickets.")
                }

                var updateDate = new Date;
                this.setLastUpdateStatus("Updated " + updateDate.formattedShortDate())
            }
        }
    },
    
    addTicketsToList : function (ticketsArray)
    {
        // Sort the tickets so that those which are waiting for
        // user response are at the top, and then sorted by modified
        // date.
        ticketsArray.sort( function (lhs, rhs) {
            var result = 0;
            
            // If they both have the same user response flag, then we sort
            // by modified date (newest to oldest) and then by title.
            if(lhs.awaitingUserResponseFlag == rhs.awaitingUserResponseFlag)
            {
                var lhsModifyDate = new Date.dateFromISO8601(lhs.modifyDate);
                var rhsModifyDate = new Date.dateFromISO8601(rhs.modifyDate);

                if(lhsModifyDate.getTime() == rhsModifyDate.getTime())
                {
                    result = lhs.title.localeCompare(rhs.title)
                } else {
                    // sort tickets with later modify dates closer to the top.
                    if(lhsModifyDate.getTime() < rhsModifyDate.getTime())
                    {
                        result = 1;
                    } else {
                        result = -1;
                    }
                }
            } else {
                // Sort tickets awaiting user response to the top.
                if(lhs.awaitingUserResponseFlag)
                {
                    result = -1;
                } else {
                    result = 1;
                }
            }

            return result;
        });
        
        for(var i = 0; i < ticketsArray.length; i++)
        {
            var ticketToAdd = ticketsArray[i]
            var ticketRow = this.createRowForTicket(ticketsArray[i])
            if(ticketRow)
            {
                document.getElementById("ticket_list").appendChild(ticketRow)
            }
        }
        
        this.refreshScrollArea()
    },
    
    // Called when one of the ticket rows is clicked.  Constructs an appropriate
    // URL for viewing the ticket and asks the system to open that URL in a browser.
    clickedRowForTicket : function (ticket)
    {
        if(ticket.serviceProvider.keyName == "THE_PLANET")
        {
            widget.openURL("https://orbit.theplanet.com/Support/OrbitTicketDetails.aspx?id=" + ticket.id % 10000000);
        } else {
            widget.openURL("https://manage.softlayer.com/Support/editTicket/" + ticket.id % 10000000);
        }
    },
    
    // Given a ticket, create a row for the table that represents that ticket.
    createRowForTicket : function (ticket)
    {
        var self = this;

        // Create a div to represent the ticket in the list.
        var ticketRow = document.createElement("div")
        ticketRow.setAttribute("class", "ticketRow " + (index % 2 ? "even" : "odd"))
        ticketRow.onclick = function (event) { self.clickedRowForTicket(ticket); }

        if(ticket.serviceProvider.keyName == "THE_PLANET")
        {
            // For the planet tickets we add a logo to identify them
            
            var planetLogoImage = document.createElement("img")
            planetLogoImage.setAttribute("src", "Images/THE_PLANET.png")
            ticketRow.appendChild(planetLogoImage)
        } else {
            // If it's not a ticket from The Planet then we add an indication
            // if the ticket is waiting for the user to respond to it.
            if(ticket.awaitingUserResponseFlag)
            {
                var customerAttentionImage = document.createElement("img")
                customerAttentionImage.setAttribute("src", "Images/CustomerAttention.png")
                customerAttentionImage.setAttribute("class", "customerAttentionImage")
                ticketRow.appendChild(customerAttentionImage)
            } else {
                var customerAttentionPlaceholder = document.createElement("div")
                customerAttentionPlaceholder.setAttribute("class", "customerAttentionImage")
                ticketRow.appendChild(customerAttentionPlaceholder)
            }
        }
        
        // The ticket details sit to the right of the icon and
        // contain things like the ticket id and title.
        var ticketDetails = document.createElement("div")
        ticketDetails.setAttribute("class", "ticketDetails")

        // The top row of the details has the ticket ID and modification date.
        var detailsTopRow = document.createElement("div")
        detailsTopRow.setAttribute("class", "detailsTopRow")
        
        var ticketNumber = document.createElement("div")
        ticketNumber.setAttribute("class", "ticketNumber")
        ticketNumber.innerText = ticket.id % 10000000;
        detailsTopRow.appendChild(ticketNumber)

        var modifiedDateLabel = document.createElement("div")
        modifiedDateLabel.setAttribute("class", "modifiedDate")

        var newDate = Date.dateFromISO8601(ticket.modifyDate)
        modifiedDateLabel.innerText = newDate.formattedShortDate()
        detailsTopRow.appendChild(modifiedDateLabel)

        // The title is in the second row of the details.
        var ticketTitle = document.createElement("div")
        ticketTitle.setAttribute("class", "ticketTitle")
        ticketTitle.innerText = ticket.title

        ticketDetails.appendChild(detailsTopRow)
        ticketDetails.appendChild(ticketTitle)

        ticketRow.appendChild(ticketDetails)

        return ticketRow
    },
    

    // Replace the contents of the ticket list with a single
    // (hopefully informative) string.
    setTicketListStatus : function (statusText)
    {
        this.clearTicketList()

        var ticketListStatus = document.createElement("div")
        ticketListStatus.setAttribute("class", "ticket_status")

        var statusTextLabel = document.createElement("span")
        statusTextLabel.setAttribute("class", "ticket_status_text")
        statusTextLabel.innerText = statusText

        ticketListStatus.appendChild(statusTextLabel)

        var content = document.getElementById("ticket_list")
        content.appendChild(ticketListStatus)

        if (scrollArea) scrollArea.object.refresh()
    },

    // Get rid of all the items (or the status text) in the ticket list
    clearTicketList : function ()
    {
        var content = document.getElementById("ticket_list")
        if (content) 
        {
            while (content.hasChildNodes()) 
            {
                content.removeChild(content.firstChild)
            }
        }

        this.refreshScrollArea()    
    },

    refreshScrollArea : function ()
    {
        var scrollArea = document.getElementById("scrollArea")
        if (scrollArea) scrollArea.object.refresh()
    },
    
    setLastUpdateStatus : function (updateString)
    {
        // Change the text of the label that indicates when the
        // list was last updated.

        var lastUpdateLabel = document.getElementById("lastUpdate");
        lastUpdateLabel.innerText = updateString;
    }
})